#include "common.h"
#include "leds.h"
#include "relays.h"

/* Edit this part to fit your project */

#define TICK_TIME_MS		100		//tick time in milliseconds
#define DEFAULT_PULSE_TIME	5		//default pulse time in ticks

static void ioInit(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;
	
	GPIO_InitStructure.GPIO_Pin             = GPIO_Pin_6 | GPIO_Pin_7 | GPIO_Pin_8 | GPIO_Pin_9;
	GPIO_InitStructure.GPIO_Speed           = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode            = GPIO_Mode_Out_PP;
	GPIO_Init(GPIOB, &GPIO_InitStructure);
	
	GPIO_InitStructure.GPIO_Pin             = GPIO_Pin_6 | GPIO_Pin_7 | GPIO_Pin_8 | GPIO_Pin_9;
	GPIO_InitStructure.GPIO_Speed           = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode            = GPIO_Mode_Out_PP;
	GPIO_Init(GPIOC, &GPIO_InitStructure);
	
}

static void ioRelSet(uint8_t id, uint8_t on)
{
	switch(id)
	{
		case 0:
			if(on) GPIO_SetBits(GPIOC,GPIO_Pin_6); else GPIO_ResetBits(GPIOC,GPIO_Pin_6);
			break;
		case 1:
			if(on) GPIO_SetBits(GPIOC,GPIO_Pin_7); else GPIO_ResetBits(GPIOC,GPIO_Pin_7);
			break;
		case 2:
			if(on) GPIO_SetBits(GPIOC,GPIO_Pin_8); else GPIO_ResetBits(GPIOC,GPIO_Pin_8);
			break;
		case 3:
			if(on) GPIO_SetBits(GPIOC,GPIO_Pin_9); else GPIO_ResetBits(GPIOC,GPIO_Pin_9);
			break;
		case 4:
			if(on) GPIO_SetBits(GPIOB,GPIO_Pin_6); else GPIO_ResetBits(GPIOB,GPIO_Pin_6);
			break;
		case 5:
			if(on) GPIO_SetBits(GPIOB,GPIO_Pin_7); else GPIO_ResetBits(GPIOB,GPIO_Pin_7);
			break;
		case 6:
			if(on) GPIO_SetBits(GPIOB,GPIO_Pin_8); else GPIO_ResetBits(GPIOB,GPIO_Pin_8);
			break;
		case 7:
			if(on) GPIO_SetBits(GPIOB,GPIO_Pin_9); else GPIO_ResetBits(GPIOB,GPIO_Pin_9);
			break;
	}
}

/* end of user-editable part*/





typedef struct {
	T_RelayAction state;
	uint8_t ticksLeft;
	uint8_t pulseDuration;
	uint16_t counter;	//internal prescaler counter
}T_Relay;



static T_Relay relay[RELNB];



void relInit(void)
{
	for(int i=0;i<RELNB;i++)
	{
		ioRelSet(i,0);
		relay[i].state = RELAY_OFF;
		relay[i].ticksLeft = 0;
		relay[i].pulseDuration = DEFAULT_PULSE_TIME;
		ioRelSet(i,0);
	}
	ioInit();
}




//should be called every 1 ms
void relProcess(void)
{
	for(int i=0;i<RELNB;i++)
	{
		if(relay[i].counter < (TICK_TIME_MS-1))
		{
			relay[i].counter++;
		}
		else
		{
			relay[i].counter = 0;
			if(relay[i].ticksLeft)
			{
				relay[i].ticksLeft--;
			}
			else
			{
				if(relay[i].state == RELAY_PULSE_ON)
				{
					ioRelSet(i,0);
					relay[i].state = RELAY_OFF;
				}
				
				if(relay[i].state == RELAY_PULSE_OFF)
				{
					ioRelSet(i,1);
					relay[i].state = RELAY_ON;
				}
			}
		}
	}//for(int i=0;i<RELNB;i++)
}


/* PARAM SET / GET */

int relSet(uint8_t relId, T_RelayAction newState)
{
	if(relId >= RELNB) return -1;
	
	switch(newState)
	{
		case RELAY_OFF:
			ioRelSet(relId,0);
			relay[relId].state = RELAY_OFF;
			relay[relId].ticksLeft = 0;
			relay[relId].counter = 0;
			return 0;
		case RELAY_ON:
			ioRelSet(relId,1);
			relay[relId].state = RELAY_ON;
			relay[relId].ticksLeft = 0;
			relay[relId].counter = 0;
			return 0;
		case RELAY_PULSE_ON:
			ioRelSet(relId,1);
			relay[relId].state = RELAY_PULSE_ON;
			relay[relId].ticksLeft = relay[relId].pulseDuration;
			relay[relId].counter = 0;
			return 0;
		case RELAY_PULSE_OFF:
			ioRelSet(relId,0);
			relay[relId].state = RELAY_PULSE_OFF;
			relay[relId].ticksLeft = relay[relId].pulseDuration;
			relay[relId].counter = 0;
			return 0;
		case RELAY_TOGGLE:
			if( (relay[relId].state==RELAY_ON) || (relay[relId].state==RELAY_PULSE_ON) )
			{
				ioRelSet(relId,0);
				relay[relId].state = RELAY_OFF;
				relay[relId].ticksLeft = 0;
				relay[relId].counter = 0;
			}
			else
			{
				ioRelSet(relId,1);
				relay[relId].state = RELAY_ON;
				relay[relId].ticksLeft = 0;
				relay[relId].counter = 0;
			}
			return 0;
	}
	
	return -2;
}


T_RelayAction relGetState(uint8_t relId)
{
	if(relId >= RELNB) return RELAY_OFF;
	
	return relay[relId].state;
}

int relSetPulseTime(uint8_t relId, uint8_t onTime)
{
	if(relId >= RELNB) return -1;
	
	relay[relId].pulseDuration = onTime;
	relay[relId].counter = 0;
	relay[relId].ticksLeft = 0;
	switch(relay[relId].state)
	{
		case RELAY_PULSE_ON:
			relay[relId].state = RELAY_OFF;
			break;
		case RELAY_PULSE_OFF:
			relay[relId].state = RELAY_ON;
			break;
		default:
			break;
	}
	
	ioRelSet(relId,0);
	
	return 0;
}

uint8_t relGetPulseTime(uint8_t relId)
{
	if(relId >= RELNB) return 0;
	
	return relay[relId].pulseDuration;
}

uint8_t relGetPulseTimeLeft(uint8_t relId)
{
	if(relId >= RELNB) return 0;
	
	return relay[relId].ticksLeft;
}
