#include "common.h"
#include "leds.h"
#include "dipsw.h"
#include "spp.h"
#include "relays.h"
#include "spp_proc.h"

/*
Template for this project is based on Digital Picture Frame project by Martin Thomas:
	http://siwawi.bauing.uni-kl.de/avr_projects/arm_projects/index_cortex.html#stm32_dpf
*/



#define USE_WATCHDOG	1

volatile uint8_t sppAddress;
T_sppPacket packet;

extern uint32_t _isr_vectorsram_offs;
extern uint32_t _isr_vectorsflash_offs;

extern volatile uint32_t tick_upd;


void init(void)
{
	SystemInit();

	//config iwdg
	#if USE_WATCHDOG
		IWDG_WriteAccessCmd(IWDG_WriteAccess_Enable);
		IWDG_SetPrescaler(IWDG_Prescaler_128);
		IWDG_SetReload(0x0AFF);
		IWDG_Enable();
		IWDG_ReloadCounter();
	#endif
	
	/* Enable USART1, GPIOA, GPIOD and AFIO clocks */
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_USART1 | RCC_APB2Periph_GPIOA | RCC_APB2Periph_GPIOB
	 | RCC_APB2Periph_GPIOC | RCC_APB2Periph_AFIO, ENABLE);
	//RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART2, ENABLE);

	#ifdef VECT_TAB_RAM
		/* vector-offset (TBLOFF) from bottom of SRAM. defined in linker script */
		
		/* Set the Vector Table base location at 0x20000000+_isr_vectorsram_offs */
		NVIC_SetVectorTable(NVIC_VectTab_RAM, (uint32_t)&_isr_vectorsram_offs);
	#else
		
		/* Set the Vector Table base location at 0x08000000+_isr_vectorsflash_offs */
		NVIC_SetVectorTable(NVIC_VectTab_FLASH, (uint32_t)&_isr_vectorsflash_offs);
	#endif /* VECT_TAB_RAM */
	
	NVIC_PriorityGroupConfig( NVIC_PriorityGroup_4 );
	
	/* Configure HCLK clock as SysTick clock source. */
	SysTick_CLKSourceConfig( SysTick_CLKSource_HCLK );
	
	ledsInit();
	
	debug_init_default();
	
	xprintf("\nSPPoB Relayz\n");
	
	/* Setup SysTick Timer */
	if ( timebase_config() ) {
		xprintf("timebase_config error");
	}
	
	RCC_ClocksTypeDef RCC_ClocksStatus;
	RCC_GetClocksFreq(&RCC_ClocksStatus);
	xprintf("Clock info: APB1 @ %d Hz, APB2 @ %d Hz\n",(int)RCC_ClocksStatus.PCLK1_Frequency,(int)RCC_ClocksStatus.PCLK2_Frequency);
	
	sppAddress = dipsw_state();
	xprintf("SPP device address = %x\n",sppAddress);
	
	sppInit();
	
	xprintf("SPP Initialized\n");
	
	relInit();
	
}




static void blink(void)
{
	static int toggle;
	static int cntr;
	if(cntr < 500)
	{
		cntr++;
	}
	else
	{
		cntr = 0;
		
		if(toggle)
		{
			toggle = 0;
			led1(0);
		}
		else
		{
			toggle = 1;
			led1(1);
		}
	}
}

int main(void)
{
	init();
	
	while(1)
	{
		
		//every 1ms
		if(tick_upd)
		{
			tick_upd = 0;
			
			relProcess();
			
			if(sppRx(&packet))
			{
				processRxedPacket(&packet);
			}
			
			sppIdleTimeout();
			
			blink();
			
			#if USE_WATCHDOG
				IWDG_ReloadCounter();
			#endif
		}
		
	}
	
	return 0;
}



